use bevy::prelude::*;

fn main() {
    App::new()
        .add_plugins(DefaultPlugins)
        .add_startup_system(setup)
        .run();
}


fn setup(mut commands: Commands) {
    commands.spawn(Camera2dBundle::default());
    commands.spawn(NodeBundle {
        background_color: Color::WHITE.into(),
        style: Style { 
            margin: UiRect::all(Val::Px(20.)),
            size: Size::new(Val::Percent(20.), Val::Px(200.)),
            align_items: AlignItems::FlexStart,
            justify_content: JustifyContent::FlexStart,
            ..default()
        },
        ..default()
    }).with_children(|parent| {
        for idx in 0..4 {
            parent.spawn(NodeBundle {
                background_color: Color::RED.into(),
                style: Style {
                    min_size: Size::all(Val::Px(40.)),
                    margin: UiRect::all(Val::Px(5.)),
                    padding: UiRect::all(Val::Px(5.)),
                    ..default()
                },
                ..default()
            }).with_children(|parent| {
                parent.spawn(NodeBundle {
                    background_color: Color::GREEN.into(),
                    style: Style {
                        size: Size::all(Val::Percent(100.)),
                        padding: UiRect::all(Val::Px(5.0)),
                        ..default()
                    },
                    ..default()
                })
                .with_children(|parent| {
                    let mut size = Size::default();
                    size.width = Val::Percent(100.);
                    if idx >= 2 {
                        // Commenting this line makes all nodes
                        // behaves the same
                        size.height = Val::Percent(100.);
                    }
                    parent.spawn(NodeBundle {
                        background_color: Color::BLUE.into(),
                        style: Style {
                            size,
                            padding: UiRect::all(Val::Px(5.)),
                            ..default()
                        },
                        ..default()
                    })
                    // Commenting this makes all nodes bahaves the same
                    // no matter what the size of blue node
                    .with_children(|parent| {
                        parent.spawn(NodeBundle::default());
                    })
                    ;
                })
                ;
            });
        }
    });
}

trait SizeAll {
    fn all(val: Val) -> Size;
}

impl SizeAll for Size {
    fn all(val: Val) -> Size {
        Size::new(val, val)
    }
}